/****************************************************************************
**
** Copyright (C) 2017 Klaralvdalens Datakonsult AB (KDAB).
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt3D module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QT3DRENDER_RENDER_COMMANDTHREAD_P_H
#define QT3DRENDER_RENDER_COMMANDTHREAD_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of other Qt classes.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtCore/QThread>
#include <QtCore/QSemaphore>
#include <QtCore/QMutex>

QT_BEGIN_NAMESPACE

class QOpenGLContext;

namespace Qt3DRender {

namespace Render {

class Renderer;
class GLCommand;
class OffscreenSurfaceHelper;
class GraphicsContext;
class ShaderCache;

class CommandThread : public QThread
{
    Q_OBJECT
public:
    explicit CommandThread(Renderer *renderer);
    ~CommandThread();

    Render::Renderer* renderer() const { return m_renderer; }

    void setShaderCache(ShaderCache *shaderCache);
    ShaderCache *shaderCache() const { return m_shaderCache; }

    void initialize(QOpenGLContext *mainContext, OffscreenSurfaceHelper *offsreenSurfaceHelper);
    void shutdown();

    void executeCommand(GLCommand *command);

private:
    void run() override;
    void executeCommandInternal(Qt3DRender::Render::GLCommand *command);

private:
    Renderer* m_renderer;
    QSemaphore m_waitForStartSemaphore;
    QSemaphore m_initializedSemaphore;
    QSemaphore m_commandRequestedSemaphore;
    QSemaphore m_commandExecutionSemaphore;
    QMutex m_blockingCallerMutex;
    QOpenGLContext *m_mainContext;
    ShaderCache *m_shaderCache;
    OffscreenSurfaceHelper *m_offsreenSurfaceHelper;
    QScopedPointer<QOpenGLContext> m_localContext;
    QScopedPointer<GraphicsContext> m_graphicsContext;
    GLCommand *m_currentCommand;
    QAtomicInt m_running;
};

} // Render

} // Qt3DRender

QT_END_NAMESPACE

#endif // QT3DRENDER_RENDER_COMMANDTHREAD_P_H
