/****************************************************************************
**
** Copyright (C) 2016 Klaralvdalens Datakonsult AB (KDAB).
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt3D module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QT3DRENDER_QPAINTEDTEXTURE_H
#define QT3DRENDER_QPAINTEDTEXTURE_H

#include <Qt3DRender/qabstracttextureimage.h>

QT_BEGIN_NAMESPACE

class QPainter;

namespace Qt3DRender {

class QPaintedTextureImagePrivate;

class QT3DRENDERSHARED_EXPORT QPaintedTextureImage : public QAbstractTextureImage
{
    Q_OBJECT
    Q_PROPERTY(int width READ width WRITE setWidth NOTIFY widthChanged)
    Q_PROPERTY(int height READ height WRITE setHeight NOTIFY heightChanged)
    Q_PROPERTY(QSize size READ size WRITE setSize NOTIFY sizeChanged)

public:
    explicit QPaintedTextureImage(Qt3DCore::QNode *parent = nullptr);
    ~QPaintedTextureImage();

    int width() const;
    int height() const;
    QSize size() const;

    void update(const QRect &rect = QRect());

public Q_SLOTS:
    void setWidth(int w);
    void setHeight(int h);
    void setSize(QSize size);

Q_SIGNALS:
    void widthChanged(int w);
    void heightChanged(int w);
    void sizeChanged(QSize size);

protected:
    virtual void paint(QPainter *painter) = 0;

private:
    Q_DECLARE_PRIVATE(QPaintedTextureImage)

    QTextureImageDataGeneratorPtr dataGenerator() const override;
};

} // namespace Qt3DRender

QT_END_NAMESPACE

#endif // QT3DRENDER_QPAINTEDTEXTURE_H
